package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysParamMapper
import com.gxzc.zen.api.sys.model.SysParam
import com.gxzc.zen.api.sys.service.ISysParamService
import com.gxzc.zen.common.contants.CACHEKEYS
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.cache.CacheManager
import org.springframework.cache.annotation.Cacheable
import org.springframework.stereotype.Service

/**
 * <p>
 * 系统参数表 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Service
class SysParamServiceImpl : ServiceImpl<SysParamMapper, SysParam>(), ISysParamService {
    companion object {
        const val CACHE_KEY_ALL = "'param_all'"
    }

    @Autowired
    private lateinit var cacheManager: CacheManager

    @Cacheable(CACHEKEYS.SYS, key = CACHE_KEY_ALL)
    override fun getListCacheable(): MutableList<SysParam> {
        return baseMapper.selectByParams(null)
    }

    override fun getList(enable: Boolean?): MutableList<SysParam> {
        return baseMapper.selectByParams(mutableMapOf(
                "enable" to enable!!
        ))
    }

    override fun getListByKey(key: String): MutableList<SysParam> {
        val allData = getListCacheable()
        return allData.filter {
            it.enable != null && it.enable!! && it.key == key
        }.toMutableList()
    }

    override fun getOneByKey(key: String, value: String?, sort: Int?): SysParam? {
        val result = baseMapper.selectByParams(mutableMapOf(
                "key" to key,
                "value" to value!!,
                "sort" to sort!!,
                "enable" to true
        ))
        return if (result.size > 0) {
            result.filter {
                it.sort == sort && it.value == value
            }[0]
        } else {
            null
        }
    }

    @Suppress("UNCHECKED_CAST")
    override fun modifySysDic(data: SysParam): SysParam {
        baseMapper.updateById(data)
        // 更新缓存
        val cache = cacheManager.getCache(CACHEKEYS.SYS)
        val cachedList: MutableList<SysParam>? = cache[CACHE_KEY_ALL].get() as MutableList<SysParam>
        cachedList?.let {
            it.removeIf { it.id == data.id }
            it.add(data)
            cache.put(CACHE_KEY_ALL, cachedList)
        }

        return data
    }

}
